@@ -29,7 +29,6 @@ static void SatEditCb(GtkWidget *, gpointer);
 static void LocEditCb(GtkWidget *, gpointer);
 static void UserEditCb(GtkWidget *, gpointer);
 
-static void Update_Sat(struct Sat_Struct *);
 static void Delete_Sat(struct Sat_Struct *);
 static void Delete_Sat_OK(void);
 
@@ -38,7 +37,18 @@ static void Delete_Loc(struct Loc_Struct *);
 static void Delete_Loc_OK(void);
 
 static GtkWidget *SatName;
-static GtkWidget *SatCat;
+static GtkWidget *SatCatAmateur;
+static GtkWidget *SatCatCommunications;
+static GtkWidget *SatCatImaging;
+static GtkWidget *SatCatGeostationary;
+static GtkWidget *SatCatIridium;
+static GtkWidget *SatCatGPS;
+static GtkWidget *SatCatHardware;
+static GtkWidget *SatCatLEO;
+static GtkWidget *SatCatMilitary;
+static GtkWidget *SatCatMolniya;
+static GtkWidget *SatCatTVRO;
+static GtkWidget *SatCatWeather;
 static GtkWidget *SatNo;
 static GtkWidget *SatEpoch;
 static GtkWidget *SatMeanAnom;
@@ -51,8 +61,9 @@ static GtkWidget *SatDrag;
 static GtkWidget *SatOrbit;
 static GtkWidget *SatAlon;
 static GtkWidget *SatAlat;
+static GtkWidget *SatZ;
 static GtkWidget *SatModes;
-static GtkWidget *SatFreq[3];
+static GtkWidget *SatFreq[2];
 
 static GtkWidget *LocName;
 static GtkWidget *LocLat;
@@ -66,7 +77,8 @@ static GtkWidget *UserHeight;
 static GtkWidget *UserTime;
 static GtkWidget *UserTrack;
 static GtkWidget *UserList;
-static GtkWidget *UserDate;
+static GtkWidget *UserDDMMYYYY;
+static GtkWidget *UserMMDDYYYY;
 
 static int Parse_Line(char *, char **);
 static int Range_Check(double, double, double);
@@ -79,15 +91,16 @@ static void SatEditCb(GtkWidget *w, gpointer data)
 	GtkWidget *window = (GtkWidget *)data;
 	struct Sat_Struct Temp_Data, *Data;
 	double Epoch;
-	char *s;
+	char *s, *s1, *s2, *p;
 	char *argv[10];
-	char *MA_Day_Name;
+	char MA_Day_Name[10], MA_Mode[10];
 	int  argc;
-	int  i;
+	int  i, len;
 	
 	memset(&Temp_Data, '\0', sizeof(struct Sat_Struct));
 
 	Temp_Data.Aflag    = FALSE;
+	Temp_Data.Minus_Z  = FALSE;
 	Temp_Data.No_Modes = 0;
 
 	s = gtk_entry_get_text(GTK_ENTRY(SatName));
@@ -103,8 +116,32 @@ static void SatEditCb(GtkWidget *w, gpointer data)
 		}
 	}
 
-	s = gtk_entry_get_text(GTK_ENTRY(SatCat));
-	strcpy(Temp_Data.Category, s);
+	strcpy(Temp_Data.Category, "");
+
+	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(SatCatAmateur)))
+		strcat(Temp_Data.Category, "A");
+	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(SatCatCommunications)))
+		strcat(Temp_Data.Category, "C");
+	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(SatCatImaging)))
+		strcat(Temp_Data.Category, "E");
+	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(SatCatGeostationary)))
+		strcat(Temp_Data.Category, "F");
+	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(SatCatIridium)))
+		strcat(Temp_Data.Category, "I");
+	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(SatCatGPS)))
+		strcat(Temp_Data.Category, "G");
+	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(SatCatHardware)))
+		strcat(Temp_Data.Category, "H");
+	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(SatCatLEO)))
+		strcat(Temp_Data.Category, "L");
+	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(SatCatMilitary)))
+		strcat(Temp_Data.Category, "M");
+	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(SatCatMolniya)))
+		strcat(Temp_Data.Category, "N");
+	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(SatCatTVRO)))
+		strcat(Temp_Data.Category, "T");
+	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(SatCatWeather)))
+		strcat(Temp_Data.Category, "W");
 
 	s = gtk_entry_get_text(GTK_ENTRY(SatNo));
 	Temp_Data.Catalogue_No = atol(s);
@@ -190,26 +227,29 @@ static void SatEditCb(GtkWidget *w, gpointer data)
 		return;
 	}
 
-	s = gtk_entry_get_text(GTK_ENTRY(SatAlon));
-	if ((argc = Parse_Line(s, argv)) != 0) {
-		Temp_Data.Alon = atof(argv[0]);
-		if (!Range_Check(Temp_Data.Alon, 0.0, 360.0)) {
-			Warning_Box("Alon must be between\n0 and 360 degrees");
-			return;
-		}
-	}
+	s1 = gtk_entry_get_text(GTK_ENTRY(SatAlon));
+	s2 = gtk_entry_get_text(GTK_ENTRY(SatAlat));
 
-	s = gtk_entry_get_text(GTK_ENTRY(SatAlat));
-	if ((argc = Parse_Line(s, argv)) != 0) {
-		Temp_Data.Alat = atof(argv[0]);
-		if (!Range_Check(Temp_Data.Alat, -90.0, 90.0)) {
-			Warning_Box("Alat must be between\n+-90 degrees");
-			return;
+	if (s1 != NULL && s2 != NULL) {
+		if ((argc = Parse_Line(s1, argv)) != 0) {
+			Temp_Data.Alon = atof(argv[0]);
+			if (!Range_Check(Temp_Data.Alon, 0.0, 360.0)) {
+				Warning_Box("Alon must be between\n0 and 360 degrees");
+				return;
+			}
 		}
-	}
 
-	if (Temp_Data.Alon != 0.0 && Temp_Data.Alat != 0.0)
-		Temp_Data.Aflag = TRUE;
+		if ((argc = Parse_Line(s2, argv)) != 0) {
+			Temp_Data.Alat = atof(argv[0]);
+			if (!Range_Check(Temp_Data.Alat, -90.0, 90.0)) {
+				Warning_Box("Alat must be between\n+90 and -90 degrees");
+				return;
+			}
+		}
+
+		Temp_Data.Minus_Z = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(SatZ));
+		Temp_Data.Aflag   = TRUE;
+	}
 
 	s = gtk_editable_get_chars(GTK_EDITABLE(SatModes), 0, -1);
 
@@ -218,7 +258,17 @@ static void SatEditCb(GtkWidget *w, gpointer data)
 	for (i = 0; i < argc; i++) {
 		Temp_Data.MA_Day[Temp_Data.No_Modes] = -1;
 
-		MA_Day_Name = strtok(argv[i], ":");
+		if ((p = strchr(argv[i], ':')) == NULL) {
+			Warning_Box("Invalid mode string, must be MA|Day:Mode");
+			return;
+		}
+
+		len = p - argv[i];
+
+		strncpy(MA_Day_Name, argv[i], len);
+		MA_Day_Name[len] = '\0';
+
+		strcpy(MA_Mode, p + 1);
 
 		if (strcasecmp(MA_Day_Name, "Sun") == 0)
 			Temp_Data.MA_Day[Temp_Data.No_Modes] = 1000;
@@ -249,7 +299,7 @@ static void SatEditCb(GtkWidget *w, gpointer data)
 			Temp_Data.MA_Day[Temp_Data.No_Modes] = atoi(MA_Day_Name);
 		}
 
-		strncpy(Temp_Data.Mode[Temp_Data.No_Modes], strtok(NULL, ":"), 4);
+		strcpy(Temp_Data.Mode[Temp_Data.No_Modes], MA_Mode);
 
 		Temp_Data.No_Modes++;
 	}
@@ -258,37 +308,33 @@ static void SatEditCb(GtkWidget *w, gpointer data)
 
 	s = gtk_entry_get_text(GTK_ENTRY(SatFreq[0]));
 	Temp_Data.Frequency1 = atof(s);
-	if (!Range_Check(Temp_Data.Frequency1, 0.0, 24000.0)) {
+	if (!Range_Check(Temp_Data.Frequency1, 0.0, 25000.0)) {
 		Warning_Box("Frequency must be between\n0 and 24GHz");
 		return;
 	}
 
 	s = gtk_entry_get_text(GTK_ENTRY(SatFreq[1]));
 	Temp_Data.Frequency2 = atof(s);
-	if (!Range_Check(Temp_Data.Frequency2, 0.0, 24000.0)) {
-		Warning_Box("Frequency must be between\n0 and 24GHz");
-		return;
-	}
-
-	s = gtk_entry_get_text(GTK_ENTRY(SatFreq[2]));
-	Temp_Data.Frequency3 = atof(s);
-	if (!Range_Check(Temp_Data.Frequency3, 0.0, 24000.0)) {
+	if (!Range_Check(Temp_Data.Frequency2, 0.0, 25000.0)) {
 		Warning_Box("Frequency must be between\n0 and 24GHz");
 		return;
 	}
 
 	if (Sat_Selection != NULL) {
 		Sat_List = g_slist_remove(Sat_List, Sat_Selection);
-		g_free(Sat_Selection);
+		Temp_Data.Display = Sat_Selection->Display;
+		Temp_Data.Current = Sat_Selection->Current;		
+		*Sat_Selection = Temp_Data;
+		Sat_List = g_slist_insert_sorted(Sat_List, Sat_Selection, Sat_Comparison);
+	} else {
+		Data  = g_malloc(sizeof(struct Sat_Struct));
+		*Data = Temp_Data;
+		Sat_List = g_slist_insert_sorted(Sat_List, Data, Sat_Comparison);
 	}
 
-	Data  = g_malloc(sizeof(struct Sat_Struct));
-	*Data = Temp_Data;
+	Write_Sat_Data();
 
-	Sat_List = g_slist_insert_sorted(Sat_List, Data, Sat_Comparison);
-
-	if (Write_Sat_Data())
-		gtk_widget_destroy(window);
+	gtk_widget_destroy(window);
 }
 
 void Add_Sat_Data(void)
@@ -301,13 +347,13 @@ void Update_Sat_Data(void)
 	Choose_Satellite(Update_Sat);
 }
 
-static void Update_Sat(struct Sat_Struct *Sat)
+void Update_Sat(struct Sat_Struct *Sat)
 {
 	GtkWidget *window;
 	GtkWidget *vbox;
 	GtkWidget *notebook;
 	GtkWidget *notelabel[3];
-	GtkWidget *table[3];
+	GtkWidget *table[4];
 	GtkWidget *label[18];
 	GtkWidget *button[3];
 	GtkWidget *bbox;
@@ -330,12 +376,51 @@ static void Update_Sat(struct Sat_Struct *Sat)
 	table[0]     = gtk_table_new(3, 2, FALSE);
 	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), table[0], notelabel[0]);
 
+	table[3] = gtk_table_new(4, 3, FALSE);
+
+	SatCatAmateur        = gtk_check_button_new_with_label("Amateur");
+	SatCatCommunications = gtk_check_button_new_with_label("Communications");
+	SatCatImaging        = gtk_check_button_new_with_label("Earth Imaging");
+	SatCatGeostationary  = gtk_check_button_new_with_label("Geostationary");
+	SatCatIridium        = gtk_check_button_new_with_label("Globalstar/Iridium");
+	SatCatGPS            = gtk_check_button_new_with_label("GPS");
+	SatCatHardware       = gtk_check_button_new_with_label("Hardware");
+	SatCatLEO            = gtk_check_button_new_with_label("LEO");
+	SatCatMilitary       = gtk_check_button_new_with_label("Military");
+	SatCatMolniya        = gtk_check_button_new_with_label("Molniya");
+	SatCatTVRO           = gtk_check_button_new_with_label("TVRO");
+	SatCatWeather        = gtk_check_button_new_with_label("Weather");
+
+	gtk_table_attach(GTK_TABLE(table[3]), SatCatAmateur,        0, 1, 0, 1,
+		GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND, BORDER_ITEM, BORDER_ITEM);
+	gtk_table_attach(GTK_TABLE(table[3]), SatCatCommunications, 0, 1, 1, 2,
+		GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND, BORDER_ITEM, BORDER_ITEM);
+	gtk_table_attach(GTK_TABLE(table[3]), SatCatImaging,        0, 1, 2, 3,
+		GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND, BORDER_ITEM, BORDER_ITEM);
+	gtk_table_attach(GTK_TABLE(table[3]), SatCatGeostationary,  0, 1, 3, 4,
+		GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND, BORDER_ITEM, BORDER_ITEM);
+	gtk_table_attach(GTK_TABLE(table[3]), SatCatIridium,        1, 2, 0, 1,
+		GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND, BORDER_ITEM, BORDER_ITEM);
+	gtk_table_attach(GTK_TABLE(table[3]), SatCatGPS,            1, 2, 1, 2,
+		GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND, BORDER_ITEM, BORDER_ITEM);
+	gtk_table_attach(GTK_TABLE(table[3]), SatCatHardware,       1, 2, 2, 3,
+		GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND, BORDER_ITEM, BORDER_ITEM);
+	gtk_table_attach(GTK_TABLE(table[3]), SatCatLEO,            1, 2, 3, 4,
+		GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND, BORDER_ITEM, BORDER_ITEM);
+	gtk_table_attach(GTK_TABLE(table[3]), SatCatMilitary,       2, 3, 0, 1,
+		GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND, BORDER_ITEM, BORDER_ITEM);
+	gtk_table_attach(GTK_TABLE(table[3]), SatCatMolniya,        2, 3, 1, 2,
+		GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND, BORDER_ITEM, BORDER_ITEM);
+	gtk_table_attach(GTK_TABLE(table[3]), SatCatTVRO,           2, 3, 2, 3,
+		GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND, BORDER_ITEM, BORDER_ITEM);
+	gtk_table_attach(GTK_TABLE(table[3]), SatCatWeather,        2, 3, 3, 4,
+		GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND, BORDER_ITEM, BORDER_ITEM);
+
 	label[0]  = gtk_label_new("Name");
 	label[1]  = gtk_label_new("Categories");
 	label[2]  = gtk_label_new("Catalogue No");
 
-	SatName   = gtk_entry_new_with_max_length(19);
-	SatCat    = gtk_entry_new_with_max_length(19);
+	SatName   = gtk_entry_new_with_max_length(50);
 	SatNo     = gtk_entry_new_with_max_length(19);
 
 	gtk_table_attach(GTK_TABLE(table[0]), label[0], 0, 1, 0, 1,
@@ -346,7 +431,7 @@ static void Update_Sat(struct Sat_Struct *Sat)
 		GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND, BORDER_ITEM, BORDER_ITEM);
 	gtk_table_attach(GTK_TABLE(table[0]), SatName,  1, 2, 0, 1,
 		GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND, BORDER_ITEM, BORDER_ITEM);
-	gtk_table_attach(GTK_TABLE(table[0]), SatCat,   1, 2, 1, 2,
+	gtk_table_attach(GTK_TABLE(table[0]), table[3], 1, 2, 1, 2,
 		GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND, BORDER_ITEM, BORDER_ITEM);
 	gtk_table_attach(GTK_TABLE(table[0]), SatNo,    1, 2, 2, 3,
 		GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND, BORDER_ITEM, BORDER_ITEM);
@@ -418,17 +503,21 @@ static void Update_Sat(struct Sat_Struct *Sat)
 
 	label[12] = gtk_label_new("Alon");
 	label[13] = gtk_label_new("Alat");
-	label[14] = gtk_label_new("Mode List");
-	label[15] = gtk_label_new("Frequency 1");
-	label[16] = gtk_label_new("Frequency 2");
-	label[17] = gtk_label_new("Frequency 3");
+	label[14] = gtk_label_new("-Z Axis");
+	label[15] = gtk_label_new("Mode List");
+	label[16] = gtk_label_new("Frequency 1");
+	label[17] = gtk_label_new("Frequency 2");
 
 	SatAlon    = gtk_entry_new_with_max_length(19);
 	SatAlat    = gtk_entry_new_with_max_length(19);
-	SatModes   = gtk_entry_new_with_max_length(19);
+	SatZ       = gtk_check_button_new();
+
+	SatModes   = gtk_text_new(NULL, NULL);
+	gtk_text_set_editable(GTK_TEXT(SatModes), TRUE);
+	gtk_text_set_word_wrap(GTK_TEXT(SatModes), TRUE);
+
 	SatFreq[0] = gtk_entry_new_with_max_length(19);
 	SatFreq[1] = gtk_entry_new_with_max_length(19);
-	SatFreq[2] = gtk_entry_new_with_max_length(19);
 
 	gtk_table_attach(GTK_TABLE(table[2]), label[12],   0, 1, 0, 1,
 		GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND, BORDER_ITEM, BORDER_ITEM);
@@ -446,13 +535,13 @@ static void Update_Sat(struct Sat_Struct *Sat)
 		GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND, BORDER_ITEM, BORDER_ITEM);
 	gtk_table_attach(GTK_TABLE(table[2]), SatAlat,     1, 2, 1, 2,
 		GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND, BORDER_ITEM, BORDER_ITEM);
-	gtk_table_attach(GTK_TABLE(table[2]), SatModes,    1, 2, 2, 3,
+	gtk_table_attach(GTK_TABLE(table[2]), SatZ,        1, 2, 2, 3,
 		GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND, BORDER_ITEM, BORDER_ITEM);
-	gtk_table_attach(GTK_TABLE(table[2]), SatFreq[0],  1, 2, 3, 4,
+	gtk_table_attach(GTK_TABLE(table[2]), SatModes,    1, 2, 3, 4,
 		GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND, BORDER_ITEM, BORDER_ITEM);
-	gtk_table_attach(GTK_TABLE(table[2]), SatFreq[1],  1, 2, 4, 5,
+	gtk_table_attach(GTK_TABLE(table[2]), SatFreq[0],  1, 2, 4, 5,
 		GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND, BORDER_ITEM, BORDER_ITEM);
-	gtk_table_attach(GTK_TABLE(table[2]), SatFreq[2],  1, 2, 5, 6,
+	gtk_table_attach(GTK_TABLE(table[2]), SatFreq[1],  1, 2, 5, 6,
 		GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND, BORDER_ITEM, BORDER_ITEM);
 
 	bbox = gtk_hbutton_box_new();
@@ -480,7 +569,30 @@ static void Update_Sat(struct Sat_Struct *Sat)
 	if (Sat != NULL) {
 		gtk_entry_set_text(GTK_ENTRY(SatName), Sat->Name);
 
-		gtk_entry_set_text(GTK_ENTRY(SatCat), Sat->Category);
+		if (strchr(Sat->Category, 'A') != NULL)
+			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(SatCatAmateur), TRUE);
+		if (strchr(Sat->Category, 'C') != NULL)
+			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(SatCatCommunications), TRUE);
+		if (strchr(Sat->Category, 'E') != NULL)
+			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(SatCatImaging), TRUE);
+		if (strchr(Sat->Category, 'F') != NULL)
+			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(SatCatGeostationary), TRUE);
+		if (strchr(Sat->Category, 'I') != NULL)
+			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(SatCatIridium), TRUE);
+		if (strchr(Sat->Category, 'G') != NULL)
+			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(SatCatGPS), TRUE);
+		if (strchr(Sat->Category, 'H') != NULL)
+			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(SatCatHardware), TRUE);
+		if (strchr(Sat->Category, 'L') != NULL)
+			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(SatCatLEO), TRUE);
+		if (strchr(Sat->Category, 'M') != NULL)
+			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(SatCatMilitary), TRUE);
+		if (strchr(Sat->Category, 'N') != NULL)
+			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(SatCatMolniya), TRUE);
+		if (strchr(Sat->Category, 'T') != NULL)
+			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(SatCatTVRO), TRUE);
+		if (strchr(Sat->Category, 'W') != NULL)
+			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(SatCatWeather), TRUE);
 
 		sprintf(buffer, "%ld", Sat->Catalogue_No);
 		gtk_entry_set_text(GTK_ENTRY(SatNo), buffer);
@@ -518,9 +630,13 @@ static void Update_Sat(struct Sat_Struct *Sat)
 
 			sprintf(buffer, "%.2f", Sat->Alat);
 			gtk_entry_set_text(GTK_ENTRY(SatAlat), buffer);
+			
+			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(SatZ), Sat->Minus_Z);
 		} else {
 			gtk_entry_set_text(GTK_ENTRY(SatAlon), "");
 			gtk_entry_set_text(GTK_ENTRY(SatAlat), "");
+
+			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(SatZ), FALSE);
 		}
 
 		*buffer = '\0';
@@ -562,7 +678,7 @@ static void Update_Sat(struct Sat_Struct *Sat)
 			strcat(buffer, " ");
 		}
 
-		gtk_entry_set_text(GTK_ENTRY(SatModes), buffer);
+		gtk_text_insert(GTK_TEXT(SatModes), NULL, NULL, NULL, buffer, -1);
 
 		if (Sat->Frequency1 != 0.0) {
 			sprintf(buffer, "%.3f", Sat->Frequency1);
@@ -577,13 +693,6 @@ static void Update_Sat(struct Sat_Struct *Sat)
 		} else {
 			gtk_entry_set_text(GTK_ENTRY(SatFreq[1]), "");
 		}
-
-		if (Sat->Frequency3 != 0.0) {
-			sprintf(buffer, "%.3f", Sat->Frequency3);
-			gtk_entry_set_text(GTK_ENTRY(SatFreq[2]), buffer);
-		} else {
-			gtk_entry_set_text(GTK_ENTRY(SatFreq[2]), "");
-		}
 	}
 
 	Sat_Selection = Sat;
@@ -592,7 +701,18 @@ static void Update_Sat(struct Sat_Struct *Sat)
 		gtk_widget_show(label[i]);
 
 	gtk_widget_show(SatName);
-	gtk_widget_show(SatCat);
+	gtk_widget_show(SatCatAmateur);
+	gtk_widget_show(SatCatCommunications);
+	gtk_widget_show(SatCatImaging);
+	gtk_widget_show(SatCatGeostationary);
+	gtk_widget_show(SatCatIridium);
+	gtk_widget_show(SatCatGPS);
+	gtk_widget_show(SatCatHardware);
+	gtk_widget_show(SatCatLEO);
+	gtk_widget_show(SatCatMilitary);
+	gtk_widget_show(SatCatMolniya);
+	gtk_widget_show(SatCatTVRO);
+	gtk_widget_show(SatCatWeather);
 	gtk_widget_show(SatNo);
 	gtk_widget_show(SatEpoch);
 	gtk_widget_show(SatIncl);
@@ -606,25 +726,26 @@ static void Update_Sat(struct Sat_Struct *Sat)
 	gtk_widget_show(SatDrag);
 	gtk_widget_show(SatAlon);
 	gtk_widget_show(SatAlat);
+	gtk_widget_show(SatZ);
 	gtk_widget_show(SatModes);
 
-	for (i = 0; i < 3; i++)
-		gtk_widget_show(SatFreq[i]);
+	gtk_widget_show(SatFreq[0]);
+	gtk_widget_show(SatFreq[1]);
 
 	for (i = 0; i < 3; i++)
 		gtk_widget_show(button[i]);
 
 	gtk_widget_show(bbox);
 
-	for (i = 0; i < 3; i++) {
+	for (i = 0; i < 4; i++)
 		gtk_widget_show(table[i]);
+
+	for (i = 0; i < 3; i++)
 		gtk_widget_show(notelabel[i]);
-	}
 
 	gtk_widget_show(notebook);
 	gtk_widget_show(vbox);
 	gtk_widget_show(window);
-
 }
 
 void Delete_Sat_Data(void)
@@ -698,16 +819,17 @@ static void LocEditCb(GtkWidget *w, gpointer data)
 
 	if (Loc_Selection != NULL) {
 		Loc_List = g_slist_remove(Loc_List, Loc_Selection);
-		g_free(Loc_Selection);
+		*Loc_Selection = Temp_Data;
+		Loc_List = g_slist_insert_sorted(Loc_List, Loc_Selection, Loc_Comparison);
+	} else {
+		Temp  = g_malloc(sizeof(struct Loc_Struct));
+		*Temp = Temp_Data;
+		Loc_List = g_slist_insert_sorted(Loc_List, Temp, Loc_Comparison);
 	}
 
-	Temp  = g_malloc(sizeof(struct Loc_Struct));
-	*Temp = Temp_Data;
+	Write_Loc_Data();
 
-	Loc_List = g_slist_insert_sorted(Loc_List, Temp, Loc_Comparison);
-
-	if (Write_Loc_Data())
-		gtk_widget_destroy(window);
+	gtk_widget_destroy(window);
 }
 
 void Add_Loc_Data(void)
@@ -863,6 +985,7 @@ static void UserEditCb(GtkWidget *w, gpointer data)
 	char Callsign[21];
 	double Latitude, Longitude, Height;
 	int Time_Interval, List_Interval, Time_Offset, Date_Format;
+	int ddmmyyyy, mmddyyyy;
 	char *s;
 	
 	s = gtk_entry_get_text(GTK_ENTRY(UserCall));
@@ -912,14 +1035,16 @@ static void UserEditCb(GtkWidget *w, gpointer data)
 		return;
 	}
 
-	s = gtk_entry_get_text(GTK_ENTRY(UserDate));
-	if (strcmp(s, "DDMMYYYY") == 0) {
+	ddmmyyyy = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(UserDDMMYYYY));
+	mmddyyyy = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(UserMMDDYYYY));
+
+	if (ddmmyyyy) {
 		Date_Format = DATE_FORMAT_DDMMYYYY;
-	} else if (strcmp(s, "MMDDYYYY") == 0) {
+	} else if (mmddyyyy) {
 		Date_Format = DATE_FORMAT_MMDDYYYY;
 	} else {
-		Warning_Box("Invalid Date Format, must be\neither DDMMYYYY or MMDDYYYY");
-		return;
+		Date_Format = DATE_FORMAT_DDMMYYYY;
+		g_error("mtrack: unknown date format, bizarre\n");
 	}
 
 	strcpy(User_Data.Callsign, Callsign);
@@ -931,8 +1056,9 @@ static void UserEditCb(GtkWidget *w, gpointer data)
 	User_Data.Time_Offset   = Time_Offset;
 	User_Data.Date_Format   = Date_Format;
 	
-	if (Write_User_Data())
-		gtk_widget_destroy(window);
+	Write_User_Data();
+
+	gtk_widget_destroy(window);
 }
 
 void Update_User_Data(void)
@@ -941,7 +1067,7 @@ void Update_User_Data(void)
 	GtkWidget *vbox;
 	GtkWidget *notebook;
 	GtkWidget *notelabel[2];
-	GtkWidget *table[2];
+	GtkWidget *table[3];
 	GtkWidget *label[8];
 	GtkWidget *button[3];
 	GtkWidget *bbox;
@@ -1003,7 +1129,16 @@ void Update_User_Data(void)
 	UserTime   = gtk_entry_new_with_max_length(10);
 	UserTrack  = gtk_entry_new_with_max_length(10);
 	UserList   = gtk_entry_new_with_max_length(10);
-	UserDate   = gtk_entry_new_with_max_length(10);
+
+	table[2] = gtk_table_new(2, 1, FALSE);
+
+	UserDDMMYYYY = gtk_radio_button_new_with_label(NULL, "DD-MM-YYYY");
+	UserMMDDYYYY = gtk_radio_button_new_with_label(gtk_radio_button_group(GTK_RADIO_BUTTON(UserDDMMYYYY)), "MM/DD/YYYY");
+
+	gtk_table_attach(GTK_TABLE(table[2]), UserDDMMYYYY, 0, 1, 0, 1,
+		GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND, BORDER_ITEM, BORDER_ITEM);
+	gtk_table_attach(GTK_TABLE(table[2]), UserMMDDYYYY, 0, 1, 1, 2,
+		GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND, BORDER_ITEM, BORDER_ITEM);
 
 	gtk_table_attach(GTK_TABLE(table[1]), label[4], 0, 1, 0, 1,
 		GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND, BORDER_ITEM, BORDER_ITEM);
@@ -1020,7 +1155,7 @@ void Update_User_Data(void)
 		GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND, BORDER_ITEM, BORDER_ITEM);
 	gtk_table_attach(GTK_TABLE(table[1]), UserList,   1, 2, 2, 3,
 		GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND, BORDER_ITEM, BORDER_ITEM);
-	gtk_table_attach(GTK_TABLE(table[1]), UserDate,   1, 2, 3, 4,
+	gtk_table_attach(GTK_TABLE(table[1]), table[2],   1, 2, 3, 4,
 		GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND, BORDER_ITEM, BORDER_ITEM);
 
 	bbox = gtk_hbutton_box_new();
@@ -1072,10 +1207,10 @@ void Update_User_Data(void)
 
 	switch (User_Data.Date_Format) {
 		case DATE_FORMAT_DDMMYYYY:
-			gtk_entry_set_text(GTK_ENTRY(UserDate), "DDMMYYYY");
+			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(UserDDMMYYYY), TRUE);
 			break;
 		case DATE_FORMAT_MMDDYYYY:
-			gtk_entry_set_text(GTK_ENTRY(UserDate), "MMDDYYYY");
+			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(UserMMDDYYYY), TRUE);
 			break;
 		default:
 			g_error("mtrack: unknown date format %d\n", User_Data.Date_Format);
@@ -1092,7 +1227,8 @@ void Update_User_Data(void)
 	gtk_widget_show(UserTime);
 	gtk_widget_show(UserList);
 	gtk_widget_show(UserTrack);
-	gtk_widget_show(UserDate);
+	gtk_widget_show(UserDDMMYYYY);
+	gtk_widget_show(UserMMDDYYYY);
 
 	for (i = 0; i < 3; i++)
 		gtk_widget_show(button[i]);
@@ -1100,6 +1236,7 @@ void Update_User_Data(void)
 	gtk_widget_show(bbox);
 	gtk_widget_show(table[0]);
 	gtk_widget_show(table[1]);
+	gtk_widget_show(table[2]);
 	gtk_widget_show(notelabel[0]);
 	gtk_widget_show(notelabel[1]);
 	gtk_widget_show(notebook);
