@@ -141,7 +141,7 @@ static int Assign_Sat_Name(char *Buffer, struct Sat_Struct *Temp_Data)
 	for (i = strlen(Buffer) - 1; Buffer[i] == ' ' && i >= 0; i--);
 	Buffer[i + 1] = '\0';
 
-	if (strlen(Buffer) > 17 || strlen(Buffer) < 2)
+	if (strlen(Buffer) > 50 || strlen(Buffer) < 2)
 		return FALSE;
 
 	strcpy(Temp_Data->Name, Buffer);
@@ -211,10 +211,10 @@ static int Insert_Sat(struct Sat_Struct New_Data)
 
 		New_Data.Category[0] = '\0';
 		New_Data.Aflag       = FALSE;
+		New_Data.Minus_Z     = FALSE;
 		New_Data.No_Modes    = 0;
 		New_Data.Frequency1  = 0.0;
 		New_Data.Frequency2  = 0.0;
-		New_Data.Frequency3  = 0.0;
 
 		Sat  = g_malloc(sizeof(struct Sat_Struct));
 		*Sat = New_Data;
@@ -226,9 +226,12 @@ static int Insert_Sat(struct Sat_Struct New_Data)
 		Sats_Updated++;
 
 		strcpy(New_Data.Category, Found->Category);
+		New_Data.Display    = Found->Display;
+		New_Data.Current    = Found->Current;
 		New_Data.Aflag      = Found->Aflag;
 		New_Data.Alon       = Found->Alon;
 		New_Data.Alat       = Found->Alat;
+		New_Data.Minus_Z    = Found->Minus_Z;
 		New_Data.No_Modes   = Found->No_Modes;
 
 		for (i = 0; i < Found->No_Modes; i++) {
@@ -238,7 +241,6 @@ static int Insert_Sat(struct Sat_Struct New_Data)
 
 		New_Data.Frequency1 = Found->Frequency1;
 		New_Data.Frequency2 = Found->Frequency2;
-		New_Data.Frequency3 = Found->Frequency3;
 
 		Sat_List = g_slist_remove(Sat_List, Found);
 		g_free(Found);
@@ -284,11 +286,22 @@ void Read_Sat_Data(void)
 			strcpy(Temp_Data.Name, Buffer + 1);
 
 			strcpy(Temp_Data.Category, "");
-			Temp_Data.Aflag      = FALSE;
-			Temp_Data.No_Modes   = 0;
-			Temp_Data.Frequency1 = 0.0;
-			Temp_Data.Frequency2 = 0.0;
-			Temp_Data.Frequency3 = 0.0;
+			Temp_Data.Aflag         = FALSE;
+			Temp_Data.Minus_Z       = FALSE;
+			Temp_Data.No_Modes      = 0;
+			Temp_Data.Frequency1    = 0.0;
+			Temp_Data.Frequency2    = 0.0;
+			
+			Temp_Data.SSP_Longitude = 0.0;
+			Temp_Data.SSP_Latitude  = 0.0;
+			Temp_Data.Half_Angle    = 0.0;
+			Temp_Data.Elevation     = 0.0;
+			Temp_Data.Range         = 0.0;
+			Temp_Data.Elapsed_Time  = 0.0;
+			
+			Temp_Data.Current       = FALSE;
+			Temp_Data.Display       = FALSE;
+			Temp_Data.Beep          = FALSE;
 		} else {
 			if ((t[0] = strtok(Buffer, "=")) == NULL)
 				continue;
@@ -318,6 +331,11 @@ void Read_Sat_Data(void)
 				Temp_Data.Epoch_Orbit = atol(t[1]);
 			} else if (strcmp(t[0], "drag") == 0) {
 				Temp_Data.Drag = atof(t[1]);
+			} else if (strcmp(t[0], "minusz") == 0) {
+				if (strcmp(t[1], "yes") == 0)
+					Temp_Data.Minus_Z = TRUE;
+				else
+					Temp_Data.Minus_Z = FALSE;
 			} else if (strcmp(t[0], "alongitude") == 0) {
 				Temp_Data.Alon  = atof(t[1]);
 				Temp_Data.Aflag = TRUE;
@@ -347,21 +365,19 @@ void Read_Sat_Data(void)
 			} else if (strcmp(t[0], "frequencies") == 0) {
 				t[2] = strtok(t[1], ",");
 				t[3] = strtok(NULL, ",");
-				t[4] = strtok(NULL, "");
 
-				if (t[2] == NULL || t[3] == NULL || t[4] == NULL)
+				if (t[2] == NULL || t[3] == NULL)
 					g_error("mtrack: cannot parse frequencies data\n");
 
 				Temp_Data.Frequency1 = atof(t[2]);
 				Temp_Data.Frequency2 = atof(t[3]);
-				Temp_Data.Frequency3 = atof(t[4]);
 			} else if (strcmp(t[0], "catalogue_no") == 0) {
 				Temp_Data.Catalogue_No = atol(t[1]);
 				Sat  = g_malloc(sizeof(struct Sat_Struct));
 				*Sat = Temp_Data;
 				Sat_List = g_slist_insert_sorted(Sat_List, Sat, Sat_Comparison);
 			} else {
-				g_error("mtrack: unknown keyword %s in %s\n", t[0], File_Name);
+				fprintf(stderr, "mtrack: unknown keyword %s in %s\n", t[0], File_Name);
 			}
 		}
 	}
@@ -404,7 +420,7 @@ void Read_Loc_Data(void)
 				*Loc = Temp_Data;
 				Loc_List = g_slist_insert_sorted(Loc_List, Loc, Loc_Comparison);
 			} else {
-				g_error("mtrack: unknown keyword %s in %s\n", t[0], File_Name);
+				fprintf(stderr, "mtrack: unknown keyword %s in %s\n", t[0], File_Name);
 			}
 		}
 	}
@@ -487,7 +503,7 @@ void Read_User_Data(void)
 					else
 						g_error("mtrack: unknown date format %s\n", t[1]);
 				} else
-					g_error("mtrack: unknown keyword %s in %s\n", t[0], File_Name);
+					fprintf(stderr, "mtrack: unknown keyword %s in %s\n", t[0], File_Name);
 				break;
 
 			case USER_COLOUR:
@@ -564,21 +580,38 @@ void Read_User_Data(void)
 					User_Data.Colours[COLOUR_USER].green = atoi(t[3]);
 					User_Data.Colours[COLOUR_USER].blue  = atoi(t[4]);
 				} else
-					g_error("mtrack: unknown keyword %s in %s\n", t[0], File_Name);
+					fprintf(stderr, "mtrack: unknown keyword %s in %s\n", t[0], File_Name);
 				break;
 
 			case USER_MAP:
 				if (strcmp(t[0], "number") == 0)
 					User_Data.Map_No = atoi(t[1]);
 				else
-					g_error("mtrack: unknown keyword %s in %s\n", t[0], File_Name);
+					fprintf(stderr, "mtrack: unknown keyword %s in %s\n", t[0], File_Name);
 				break;
 
 			case USER_STARTUP:
-				if (strcmp(t[0], "satellite") == 0)
-					strcpy(User_Data.Startup, t[1]);
-				else
-					g_error("mtrack: unknown keyword %s in %s\n", t[0], File_Name);
+				if (strcmp(t[0], "satellite") == 0) {
+					struct Sat_Struct *Sat_Data;
+
+					if ((Sat_Data = Find_Satellite_Name(t[1])) != NULL) {
+						if (Num_Chosen < MAX_CHOSEN) {
+							Sat_Data->Display = TRUE;
+							Num_Chosen++;
+						}
+					} else {
+						g_error("mtrack: unknown satellite name %s in %s\n", t[1], File_Name);
+					}
+				} else if (strcmp(t[0], "current") == 0) {
+					struct Sat_Struct *Sat_Data;
+
+					if ((Sat_Data = Find_Satellite_Name(t[1])) != NULL) {
+						Sat_Data->Current = TRUE;
+					} else {
+						g_error("mtrack: unknown satellite name %s in %s\n", t[1], File_Name);
+					}
+				} else
+					fprintf(stderr, "mtrack: unknown keyword %s in %s\n", t[0], File_Name);
 				break;
 
 			default:
@@ -607,7 +640,7 @@ gint Loc_Comparison(gconstpointer d1, gconstpointer d2)
 	return strcasecmp(First->QTH, Second->QTH);
 }
 
-int Write_Sat_Data(void)
+void Write_Sat_Data(void)
 {
 	char File_Name[81];
 	GSList *Sat;
@@ -618,8 +651,8 @@ int Write_Sat_Data(void)
 	sprintf(File_Name, "%s/satfile", Files_Path);
 
 	if ((fp = fopen(File_Name, "w")) == NULL) {
-		Error_Box("Cannot open satfile for writing");
-		return FALSE;
+		Warning_Box("Cannot open satfile for writing, changes are not saved");
+		return;
 	}
 
 	Sat = Sat_List;
@@ -645,6 +678,9 @@ int Write_Sat_Data(void)
 			fprintf(fp, "alatitude=%f\n", Data->Alat);
 		}
 
+		if (Data->Minus_Z)
+			fprintf(fp, "minusz=yes\n");
+
 		if (Data->No_Modes > 0) {
 			fprintf(fp, "modes=");
 
@@ -654,8 +690,8 @@ int Write_Sat_Data(void)
 			fprintf(fp, "\n");
 		}
 
-		if (Data->Frequency1 != 0.0 || Data->Frequency2 != 0.0 || Data->Frequency3 != 0.0)
-			fprintf(fp, "frequencies=%f,%f,%f\n", Data->Frequency1, Data->Frequency2, Data->Frequency3);
+		if (Data->Frequency1 != 0.0 || Data->Frequency2 != 0.0)
+			fprintf(fp, "frequencies=%f,%f\n", Data->Frequency1, Data->Frequency2);
 
 		fprintf(fp, "catalogue_no=%ld\n\n", Data->Catalogue_No);
 
@@ -663,11 +699,9 @@ int Write_Sat_Data(void)
 	}
 
 	fclose(fp);
-
-	return TRUE;
 }
 
-int Write_Loc_Data(void)
+void Write_Loc_Data(void)
 {
 	char File_Name[81];
 	GSList *Loc;
@@ -677,8 +711,8 @@ int Write_Loc_Data(void)
 	sprintf(File_Name, "%s/locfile", Files_Path);
 
 	if ((fp = fopen(File_Name, "w")) == NULL) {
-		Error_Box("Cannot open locfile for writing");
-		return FALSE;
+		Warning_Box("Cannot open locfile for writing, changes are not saved");
+		return;
 	}
 
 	Loc = Loc_List;
@@ -695,29 +729,29 @@ int Write_Loc_Data(void)
 	}
 
 	fclose(fp);
-
-	return TRUE;
 }
 
-int Write_User_Data(void)
+void Write_User_Data(void)
 {
 	char File_Name[81], *s;
+	struct Sat_Struct *Sat;
+	GSList *Sat_Iterator;
 	FILE *fp;
 
 	if (getuid() == 0) {
 		sprintf(File_Name, "%s/mtrackrc", RCDIR);
 	} else {
 		if ((s = getenv("HOME")) == NULL) {
-			Error_Box("Cannot find home directory");
-			return FALSE;
+			Warning_Box("Cannot find home directory, changes not saved");
+			return;
 		}
 
 		sprintf(File_Name, "%s/.mtrackrc", s);
 	}
 
 	if ((fp = fopen(File_Name, "w")) == NULL) {
-		Error_Box("Cannot open userfile for writing");
-		return FALSE;
+		Warning_Box("Cannot open userfile for writing, changes not saved");
+		return;
 	}
 
 	fprintf(fp, "[User]\n");
@@ -750,12 +784,23 @@ int Write_User_Data(void)
 	fprintf(fp, "\n[Map]\n");
 	fprintf(fp, "number=%d\n", User_Data.Map_No);
 
-	if (Sat_Chosen != NULL) {
+	if (Num_Chosen > 0) {
 		fprintf(fp, "\n[Startup]\n");
-		fprintf(fp, "satellite=%s\n", Sat_Chosen->Name);
+
+		Sat_Iterator = Sat_List;
+
+		while (Sat_Iterator != NULL) {
+			Sat = (struct Sat_Struct *)Sat_Iterator->data;
+		
+			if (Sat->Display)
+				fprintf(fp, "satellite=%s\n", Sat->Name);
+			
+			Sat_Iterator = g_slist_next(Sat_Iterator);
+		}
+
+		if ((Sat = Find_Current()) != NULL)
+			fprintf(fp, "current=%s\n", Sat->Name);
 	}
 
 	fclose(fp);
-
-	return TRUE;
 }
