@@ -32,14 +32,13 @@
 #define	MESSAGE_BOX	2
 #define	INFO_BOX	3
 #define	YN_BOX		4
-#define	EXIT_BOX	5
 
-char *Files_Path              = VARDIR;
+char *Files_Path = VARDIR;
 
-GSList *Sat_List              = NULL;
-struct Sat_Struct *Sat_Chosen = NULL;
+GSList *Sat_List = NULL;
+int Num_Chosen   = 0;
 
-GSList *Loc_List              = NULL;
+GSList *Loc_List = NULL;
 
 struct User_Struct User_Data = {
 	"G4KLX",
@@ -56,9 +55,8 @@ struct User_Struct User_Data = {
 	{0, 65535, 0, 0},
 	{0, 0, 0, 0},
 	{0, 65535, 0, 0},
-	{0, 65535, 65535, 0}
+	{0, 65535, 65535, 0},
 	},
-	"",		/* Startup Satellite */
 	DATE_FORMAT_DDMMYYYY
 };
 
@@ -71,16 +69,14 @@ GtkWidget *MapText;
 static GtkWidget *List_Ask_Entry[4];
 static void ListAskCb(GtkWidget *, GtkWidget *);
 
-static void Exit_Box(void);
+static gint TimeoutCb(gpointer);
 static void ExitCb(GtkWidget *, GtkWidget *);
 
-static gint TimeoutCb(gpointer);
-
 static void Track_Sat(struct Sat_Struct *);
-static void List_Sat(void);
-
-static int Time_Offset;
 
+static gint Timeout_Tag;
+static int  Time_Offset;
+static time_t UTCoffset;
 static void General_Box(char *, char *, int);
 
 static void YesCb(GtkWidget *, GtkWidget *);
@@ -89,49 +85,50 @@ static void (*Question_Yes)(void);
 static void (*Question_No)(void);
 
 static GtkItemFactoryEntry Menu_Items[] = {
-	{"/_File",                      NULL,      NULL,     0, "<Branch>"},
-	{"/File/New",                   NULL,      FileCb,   0, NULL},
-	{"/File/sep",                   NULL,      NULL,     0, "<Separator>"},
-	{"/File/Tracking Table",        NULL,      FileCb,   1, NULL},
-	{"/File/Updated Elements",      NULL,      FileCb,   2, NULL},
-	{"/File/Visibility",            NULL,      FileCb,   3, NULL},
-	{"/File/sep",                   NULL,      NULL,     0, "<Separator>"},
-	{"/File/Exit",                  NULL,      FileCb,   4, NULL},
-	{"/_Edit",                      NULL,      NULL,     0, "<Branch>"},
-	{"/Edit/Satellite",             NULL,      NULL,     0, "<Branch>"},
-	{"/Edit/Satellite/Add",         NULL,      EditCb,   0, NULL},
-	{"/Edit/Satellite/Delete",      NULL,      EditCb,   1, NULL},
-	{"/Edit/Satellite/Modify",      NULL,      EditCb,   2, NULL},
-	{"/Edit/Satellite/Import TLE",  NULL,      EditCb,   3, NULL},
-	{"/Edit/Locations",             NULL,      NULL,     0, "<Branch>"},
-	{"/Edit/Locations/Add",         NULL,      EditCb,   4, NULL},
-	{"/Edit/Locations/Delete",      NULL,      EditCb,   5, NULL},
-	{"/Edit/Locations/Modify",      NULL,      EditCb,   6, NULL},
-	{"/Edit/Preferences",           NULL,      EditCb,   7, NULL},
-	{"/Edit/Colours",               NULL,      NULL,     0, "<Branch>"},
-	{"/Edit/Colours/Background",    NULL,      ColourCb, COLOUR_BACKGROUND, NULL},
-	{"/Edit/Colours/Cities",        NULL,      ColourCb, COLOUR_CITIES, NULL},
-	{"/Edit/Colours/Footprint",     NULL,      ColourCb, COLOUR_FOOTPRINT, NULL},
-	{"/Edit/Colours/Land",          NULL,      ColourCb, COLOUR_LAND, NULL},
-	{"/Edit/Colours/Satellite",     NULL,      ColourCb, COLOUR_SATELLITE, NULL},
-	{"/Edit/Colours/User",          NULL,      ColourCb, COLOUR_USER, NULL},
-	{"/_Map",                       NULL,      NULL,     0, "<Branch>"},
-	{"/Map/Africa",                 NULL,      MapCb,    0, NULL},
-	{"/Map/Asia",                   NULL,      MapCb,    1, NULL},
-	{"/Map/Australasia",            NULL,      MapCb,    2, NULL},
-	{"/Map/Europe",                 NULL,      MapCb,    3, NULL},
-	{"/Map/N America",              NULL,      MapCb,    4, NULL},
-	{"/Map/Pacific",                NULL,      MapCb,    5, NULL},
-	{"/Map/S America",              NULL,      MapCb,    6, NULL},
-	{"/Map/USA",                    NULL,      MapCb,    7, NULL},
-	{"/Map/World",                  NULL,      MapCb,    8, NULL},
-	{"/_Help",                      NULL,      NULL,     0, "<LastBranch>"},
-	{"/Help/Mtrack Help",           NULL,      HelpCb,   1, NULL},
-	{"/Help/Contents and Index",    NULL,      HelpCb,   0, NULL},
-	{"/Help/sep",                   NULL,      NULL,     0, "<Separator>"},
-	{"/Help/About Mtrack",          NULL,      AboutCb,  0, NULL}
+	{"/_File",                         NULL,      NULL,     0, "<Branch>"},
+	{"/File/_New...",                  NULL,      FileCb,   0, NULL},
+	{"/File/sep",                      NULL,      NULL,     0, "<Separator>"},
+	{"/File/_Tracking Table...",       NULL,      FileCb,   1, NULL},
+	{"/File/_Updated Elements",        NULL,      FileCb,   2, NULL},
+	{"/File/_Visibility",              NULL,      FileCb,   3, NULL},
+	{"/File/sep",                      NULL,      NULL,     0, "<Separator>"},
+	{"/File/_Exit",                    NULL,      FileCb,   4, NULL},
+	{"/_Edit",                         NULL,      NULL,     0, "<Branch>"},
+	{"/Edit/_Satellite",               NULL,      NULL,     0, "<Branch>"},
+	{"/Edit/Satellite/_Add...",        NULL,      EditCb,   0, NULL},
+	{"/Edit/Satellite/_Delete...",     NULL,      EditCb,   1, NULL},
+	{"/Edit/Satellite/_Modify...",     NULL,      EditCb,   2, NULL},
+	{"/Edit/Satellite/_Import TLE...", NULL,      EditCb,   3, NULL},
+	{"/Edit/_Locations",               NULL,      NULL,     0, "<Branch>"},
+	{"/Edit/Locations/_Add...",        NULL,      EditCb,   4, NULL},
+	{"/Edit/Locations/_Delete...",     NULL,      EditCb,   5, NULL},
+	{"/Edit/Locations/_Modify...",     NULL,      EditCb,   6, NULL},
+	{"/Edit/_Preferences...",          NULL,      EditCb,   7, NULL},
+	{"/Edit/_Colours",                 NULL,      NULL,     0, "<Branch>"},
+	{"/Edit/Colours/_Background...",   NULL,      ColourCb, COLOUR_BACKGROUND, NULL},
+	{"/Edit/Colours/_Cities...",       NULL,      ColourCb, COLOUR_CITIES, NULL},
+	{"/Edit/Colours/_Footprint...",    NULL,      ColourCb, COLOUR_FOOTPRINT, NULL},
+	{"/Edit/Colours/_Land...",         NULL,      ColourCb, COLOUR_LAND, NULL},
+	{"/Edit/Colours/_Satellite...",    NULL,      ColourCb, COLOUR_SATELLITE, NULL},
+	{"/Edit/Colours/_User...",         NULL,      ColourCb, COLOUR_USER, NULL},
+	{"/_Map",                          NULL,      NULL,     0, "<Branch>"},
+	{"/Map/Africa",                    NULL,      MapCb,    0, NULL},
+	{"/Map/Asia",                      NULL,      MapCb,    1, NULL},
+	{"/Map/Australasia",               NULL,      MapCb,    2, NULL},
+	{"/Map/Europe",                    NULL,      MapCb,    3, NULL},
+	{"/Map/N America",                 NULL,      MapCb,    4, NULL},
+	{"/Map/Pacific",                   NULL,      MapCb,    5, NULL},
+	{"/Map/S America",                 NULL,      MapCb,    6, NULL},
+	{"/Map/USA",                       NULL,      MapCb,    7, NULL},
+	{"/Map/_World",                    NULL,      MapCb,    8, NULL},
+	{"/_Help",                         NULL,      NULL,     0, "<LastBranch>"},
+	{"/Help/Mtrack Help",              NULL,      HelpCb,   1, NULL},
+	{"/Help/Contents and Index",       NULL,      HelpCb,   0, NULL},
+	{"/Help/sep",                      NULL,      NULL,     0, "<Separator>"},
+	{"/Help/About Mtrack",             NULL,      AboutCb,  0, NULL}
 };
 
+
 int main(int argc, char **argv)
 {
 	GtkWidget *window;
@@ -140,9 +137,15 @@ int main(int argc, char **argv)
 	GtkWidget *separator;
 	GtkItemFactory *item_factory;
 	GtkAccelGroup *accel_group;
-	struct Sat_Struct *Sat_Data;
 	int nmenu_items = sizeof(Menu_Items) / sizeof(Menu_Items[0]);
 	char *s;
+	time_t t;
+	struct tm *tm;
+
+	time(&t);
+	tm = gmtime(&t);
+	tm->tm_isdst = -1; /* gmtime() may not set isdst right. We force to "don't know" */
+	UTCoffset = mktime(tm) - t; /* positive if W of Greenwich */
 
 	gtk_init(&argc, &argv);
 
@@ -169,6 +172,8 @@ int main(int argc, char **argv)
 	MapArea = gtk_drawing_area_new();
 	gtk_drawing_area_size(GTK_DRAWING_AREA(MapArea), MAP_WIDTH, MAP_HEIGHT);
 	gtk_signal_connect(GTK_OBJECT(MapArea), "expose_event", (GtkSignalFunc)MapAreaCb, NULL);
+	gtk_signal_connect(GTK_OBJECT(MapArea), "button_press_event", GTK_SIGNAL_FUNC(maparea_buttonpress_cb), NULL);
+	gtk_widget_set_events(MapArea, GDK_EXPOSURE_MASK | GDK_BUTTON_PRESS_MASK );
 	gtk_box_pack_start(GTK_BOX(vbox), MapArea, FALSE, TRUE, 0);
 	gtk_widget_show(MapArea);
 
@@ -193,13 +198,10 @@ int main(int argc, char **argv)
 
 	Read_User_Data();
 
-	if ((Sat_Data = Find_Satellite_Name(User_Data.Startup)) != NULL)
-		Track_Sat(Sat_Data);
-
-	gtk_timeout_add(User_Data.Time_Interval * 1000L, TimeoutCb, NULL);
-
 	gtk_widget_show(window);
 
+	Timeout_Tag = gtk_timeout_add(User_Data.Time_Interval * 1000L, TimeoutCb, NULL);
+
 	gtk_main();
 
 	return 0;
@@ -208,13 +210,21 @@ int main(int argc, char **argv)
 static gint TimeoutCb(gpointer data)
 {
 	time_t Time_Now;
-	struct tm *Tm_Time_Now;
+	static int Last_Interval = 3600;
+
+	/*
+	 * If the user has changed the time interval, or it is the first time
+	 * through ...
+	 */
+	if (User_Data.Time_Interval != Last_Interval) {
+		gtk_timeout_remove(Timeout_Tag);
+		Timeout_Tag = gtk_timeout_add(User_Data.Time_Interval * 1000L, TimeoutCb, NULL);
+		Last_Interval = User_Data.Time_Interval;
+	}
+
+	if (Num_Chosen > 0) {
+		time(&Time_Now);   /* get UTC time */
 
-	if (Sat_Chosen != NULL) {
-		time(&Time_Now);
-		Tm_Time_Now = gmtime(&Time_Now);
-		Time_Now    = mktime(Tm_Time_Now);
-		
 		Track_Satellite(Time_Now + Time_Offset);
 	}
 
@@ -223,32 +233,40 @@ static gint TimeoutCb(gpointer data)
 
 static void FileCb(GtkWidget *w, gpointer data)
 {
+	struct Sat_Struct *Sat;
 	int n = (int)data;
-	
+
 	switch (n) {
 		case 0:
 			Choose_Satellite(Track_Sat);
 			break;
 		case 1:
-			if (Sat_Chosen != NULL)
-				List_Sat();
+			Sat = Find_Current();
+
+			if (Sat != NULL)
+				List_Sat(Sat);
 			else
 				Warning_Box("No satellite selected yet");
 			break;
 		case 2:
-			if (Sat_Chosen != NULL)
-				Show_Elements();
+			Sat = Find_Current();
+
+			if (Sat != NULL)
+				Show_Elements(Sat);
 			else
 				Warning_Box("No satellite selected yet");
 			break;
 		case 3:
-			if (Sat_Chosen != NULL)
-				Show_Locations();
+			Sat = Find_Current();
+
+			if (Sat != NULL)
+				Show_Locations(Sat);
 			else
 				Warning_Box("No satellite selected yet");
 			break;
 		case 4:
-			Exit_Box();
+			Write_User_Data();
+			gtk_main_quit();
 			break;
 		default:
 			break;
@@ -257,9 +275,38 @@ static void FileCb(GtkWidget *w, gpointer data)
 
 static void Track_Sat(struct Sat_Struct *Sat)
 {
-	Sat_Chosen = Sat;
+	struct Sat_Struct *Data;
+	time_t Time_Now;
 
-	Time_Offset = User_Data.Time_Offset * 3600;
+	if (Num_Chosen == MAX_CHOSEN) {
+		Warning_Box("Displayed satellite limit has\nbeen reached.");
+		return;
+	}
+
+	/* It is already selected */
+	if (Sat->Display)
+		return;
+
+	/* Remove any other Current flags */
+	if ((Data = Find_Current()) != NULL)
+		Data->Current = FALSE;
+
+	Sat->SSP_Longitude = 0.0;
+	Sat->SSP_Latitude  = 0.0;
+	Sat->Half_Angle    = 0.0;
+	Sat->Elevation     = 0.0;
+	Sat->Range         = 0.0;
+	Sat->Elapsed_Time  = 0.0;
+	
+	Sat->Current       = TRUE;
+	Sat->Display       = TRUE;
+	Sat->Beep          = FALSE;
+
+	Num_Chosen++;
+
+	time(&Time_Now);   /* get UTC time */
+
+	Track_Satellite(Time_Now + Time_Offset);
 }
 
 static void EditCb(GtkWidget *w, gpointer data)
@@ -355,19 +402,6 @@ static void General_Box(char *title, char *message, int type)
 			gtk_widget_show(button[0]);
 			gtk_widget_show(button[1]);
 			break;
-		case EXIT_BOX:
-			button[0] = gtk_button_new_with_label("OK");
-			button[1] = gtk_button_new_with_label("Cancel");
-			gtk_container_add(GTK_CONTAINER(bbox), button[0]);
-			gtk_container_add(GTK_CONTAINER(bbox), button[1]);
-			gtk_signal_connect(GTK_OBJECT(button[0]), "clicked", ExitCb, window);
-			gtk_signal_connect_object(GTK_OBJECT(button[1]), "clicked", GTK_SIGNAL_FUNC(gtk_widget_destroy), GTK_OBJECT(window));
-			GTK_WIDGET_SET_FLAGS(button[0], GTK_CAN_DEFAULT);
-			GTK_WIDGET_SET_FLAGS(button[1], GTK_CAN_DEFAULT);
-			gtk_widget_grab_default(button[0]);
-			gtk_widget_show(button[0]);
-			gtk_widget_show(button[1]);
-			break;
 		default:
 			g_error("mtrack: unknown box type %d\n", type);
 			break;
@@ -424,11 +458,6 @@ static void NoCb(GtkWidget *widget, GtkWidget *window)
 		(*Question_No)();
 }
 
-void Exit_Box(void)
-{
-	General_Box("Exit Mtrack", "Leave Mtrack ?", EXIT_BOX);
-}
-
 static void ExitCb(GtkWidget *widget, GtkWidget *window)
 {
 	Write_User_Data();
@@ -438,7 +467,9 @@ static void ExitCb(GtkWidget *widget, GtkWidget *window)
 	gtk_main_quit();
 }
 
-static void List_Sat(void)
+static struct Sat_Struct *Sat;
+
+void List_Sat(struct Sat_Struct *Sat_Chosen)
 {
 	GtkWidget *window;
 	GtkWidget *hbox;
@@ -447,17 +478,21 @@ static void List_Sat(void)
 	GtkWidget *button[3];
 	GtkWidget *frame;
 	GtkWidget *bbox;
-	char Buffer[21];
+	char Buffer[40];
 	time_t Time_Now;
 	struct tm *tm;
 	int i;
-	
+
 	time(&Time_Now);
 	tm = gmtime(&Time_Now);
 
+	Sat = Sat_Chosen;
+
 	window = gtk_window_new(GTK_WINDOW_DIALOG);
 
-	gtk_window_set_title(GTK_WINDOW(window), "Track List");
+	sprintf(Buffer, "Tracking List for %s", Sat_Chosen->Name);
+
+	gtk_window_set_title(GTK_WINDOW(window), Buffer);
 	gtk_container_set_border_width(GTK_CONTAINER(window), BORDER_WIDTH);
 
 	hbox = gtk_hbox_new(FALSE, BORDER_WIDTH);
@@ -632,7 +667,7 @@ static void ListAskCb(GtkWidget *widget, GtkWidget *window)
 	tm.tm_min  = Minutes;
 	tm.tm_sec  = Seconds;
 	tm.tm_isdst= FALSE;
-	Start = mktime(&tm);
+	Start = mktime(&tm) - UTCoffset;  /* compensate for mktime()'s TZ correction */
 
 	s = gtk_entry_get_text(GTK_ENTRY(List_Ask_Entry[1]));
 	switch (User_Data.Date_Format) {
@@ -683,7 +718,7 @@ static void ListAskCb(GtkWidget *widget, GtkWidget *window)
 	tm.tm_min  = Minutes;
 	tm.tm_sec  = Seconds;
 	tm.tm_isdst= FALSE;
-	End = mktime(&tm);
+	End = mktime(&tm) - UTCoffset;  /* compensate for mktime()'s TZ correction */
 
 	if (Start >= End) {
 		Warning_Box("End Time must be later than the\nStart Time");
@@ -706,5 +741,5 @@ static void ListAskCb(GtkWidget *widget, GtkWidget *window)
 
 	gtk_widget_destroy(window);
 
-	List_Satellite(Start, End, Step, Elevation);
+	List_Satellite(Sat, Start, End, Step, Elevation);
 }
