@@ -45,6 +45,8 @@ static double RAAN;
 static int    MA;
 static char   Mode[6];
 static double Squint;
+static time_t Next_AOS = 0;  /* 0 if not available */
+static time_t Next_LOS = 0;  /* 0 if not available */
 
 static double Precession;
 static double Station_Earth_X;
@@ -53,25 +55,54 @@ static double Station_Earth_Z;
 static double Sidereal_Epoch;
 static double Sidereal_Constant;
 
-static void Calculate_Initial_Data(void);
-static void Calculate_Times(long);
-static void Calculate_Satellite_Position(void);
-static void Display_Track_Data(void);
-static void Display_List_Data(GtkWidget *);
+static void Calculate_Initial_Data(struct Sat_Struct *);
+static void Calculate_Times(struct Sat_Struct *, long);
+static void Calculate_Satellite_Position(struct Sat_Struct *);
+static void Display_Track_Data(struct Sat_Struct *, time_t);
+static void Display_List_Data(struct Sat_Struct *, GtkWidget *);
 static int  Year_Conversion(int);
+static void Search_AOS_LOS(struct Sat_Struct *, time_t);
 
 void Track_Satellite(time_t Track_Time)
 {
-	Calculate_Initial_Data();
+	struct Sat_Struct *Sat;
+	GSList *Sat_Iterator;
 
-	Calculate_Times(Track_Time);
+	Sat_Iterator = Sat_List;
 
-	Calculate_Satellite_Position();
+	while (Sat_Iterator != NULL) {
+		Sat = (struct Sat_Struct *)Sat_Iterator->data;
 
-	Display_Track_Data();
+		if (Sat->Display) {
+			Calculate_Initial_Data(Sat);
+
+			if (Sat->Current) Search_AOS_LOS(Sat, Track_Time);
+
+			Calculate_Times(Sat, Track_Time);
+			Calculate_Satellite_Position(Sat);
+
+			if (Elevation >= 0.0 && Sat->Elevation < 0.0 && Sat->Beep)
+				gdk_beep();
+
+			if (Sat->Current) Display_Track_Data(Sat, Track_Time);
+
+			Sat->SSP_Longitude = SSP_Long;
+			Sat->SSP_Latitude  = SSP_Lat;
+			Sat->Half_Angle    = Half_Angle;
+			Sat->Elevation     = Elevation;
+			Sat->Range         = Range;
+			Sat->Elapsed_Time  = Elapsed_Time;
+		}
+
+		Sat_Iterator = g_slist_next(Sat_Iterator);
+	}
+
+	Refresh_Text();
+
+	Redraw_Satellites();
 }
 
-void List_Satellite(time_t Start, time_t End, int Step, double Min_Elevation)
+void List_Satellite(struct Sat_Struct *Sat, time_t Start, time_t End, int Step, double Min_Elevation)
 {
 	GtkWidget *window;
 	GtkWidget *vbox;
@@ -80,11 +111,14 @@ void List_Satellite(time_t Start, time_t End, int Step, double Min_Elevation)
 	GtkWidget *bbox;
 	GtkWidget *button[2];
 	char *title[10];
+	char Buffer[50];
 	time_t Track_Time;
 
 	window = gtk_window_new(GTK_WINDOW_DIALOG);
 
-	gtk_window_set_title(GTK_WINDOW(window), "Tabular Track");
+	sprintf(Buffer, "Tabular Track for %s", Sat->Name);
+
+	gtk_window_set_title(GTK_WINDOW(window), Buffer);
 	gtk_container_set_border_width(GTK_CONTAINER(window), BORDER_WIDTH);
 
 	vbox = gtk_vbox_new(FALSE, BORDER_WIDTH);
@@ -133,15 +167,14 @@ void List_Satellite(time_t Start, time_t End, int Step, double Min_Elevation)
 
 	gtk_clist_freeze(GTK_CLIST(listlist));
 
-	Calculate_Initial_Data();
+	Calculate_Initial_Data(Sat);
 
 	for (Track_Time = Start; Track_Time <= End; Track_Time += Step) {
-		Calculate_Times(Track_Time);
-
-		Calculate_Satellite_Position();
+		Calculate_Times(Sat, Track_Time);
+		Calculate_Satellite_Position(Sat);
 
 		if (Elevation >= Min_Elevation)
-			Display_List_Data(listlist);
+			Display_List_Data(Sat, listlist);
 	}
 
 	gtk_clist_thaw(GTK_CLIST(listlist));
@@ -156,7 +189,7 @@ void List_Satellite(time_t Start, time_t End, int Step, double Min_Elevation)
 	gtk_widget_show(window);
 }
 
-static void Calculate_Initial_Data(void)
+static void Calculate_Initial_Data(struct Sat_Struct *Sat_Chosen)
 {
 	double Earth_Radius;
 	double Eccentricity;
@@ -175,7 +208,7 @@ static void Calculate_Initial_Data(void)
 
 	SMA = R * pow(107.08816036 / (2.0 * PI * Mean_Motion), 2.0 / 3.0);
 
-	Precession = 9.95 * pow(R / SMA, 3.5) / pow(1.0 - pow(Eccentricity, 2.0), 2.0);
+	Precession = 9.964 * pow(R / SMA, 3.5) / pow(1.0 - pow(Eccentricity, 2.0), 2.0);
 
 	Earth_Radius = R * (1.0 - (F / 2.0) * (cos(2.0 * Latitude) - 1.0)) + QTH_Height / 1000.0;
 
@@ -186,7 +219,7 @@ static void Calculate_Initial_Data(void)
 	Station_Earth_Y = Earth_Radius * cos(L8) * sin(-Longitude);
 }
 
-static void Calculate_Times(time_t Track_Time)
+static void Calculate_Times(struct Sat_Struct *Sat_Chosen, time_t Track_Time)
 {
 	double Epoch_Day_Time;
 	struct tm *Time_Struct;
@@ -195,11 +228,9 @@ static void Calculate_Times(time_t Track_Time)
 	Time_Struct = gmtime(&Track_Time);
 
 	Current_Time = *Time_Struct;
-	Current_Time.tm_mon++;
-	Current_Time.tm_yday++;
 
 	Track_Day_Time = (double)Year_Conversion(Current_Time.tm_year + 1900) +
-			 (double)Current_Time.tm_yday +
+			 (double)(Current_Time.tm_yday + 1) +
 			 (double)Current_Time.tm_hour / 24.0  +
 			 (double)Current_Time.tm_min / 1440.0 +
 			 (double)Current_Time.tm_sec / (60.0 * 1440.0);
@@ -215,16 +246,16 @@ static void Calculate_Times(time_t Track_Time)
 	Sidereal_Constant = (6.6460656 + Temp * (2400.051262 + Temp * 2.581e-5)) / 24.0 - (double)Current_Time.tm_year;
 }
 
-static void Calculate_Satellite_Position(void)
+static void Calculate_Satellite_Position(struct Sat_Struct *Sat_Chosen)
 {
 	double Earth_X, Earth_Y, Earth_Z;
-	double Orbit_X, Orbit_Y;
+	double Orbit_X, Orbit_Y, Orbit_Z;
 	double Stars_X, Stars_Y, Stars_Z;
 	double Station_X, Station_Y, Station_Z;
 	double X8, Y8, Z8;
 	double Alat;
 	double Alon;
-	double C[3][2];
+	double C[3][3];
 	double Denom;
 	double Drag;
 	double Drag_Term;
@@ -262,12 +293,15 @@ static void Calculate_Satellite_Position(void)
 
 	C[0][0] =  cos(Arg_Of_Perigee) * cos(RAAN) - sin(Arg_Of_Perigee) * sin(RAAN) * cos(Inclination);
 	C[0][1] = -sin(Arg_Of_Perigee) * cos(RAAN) - cos(Arg_Of_Perigee) * sin(RAAN) * cos(Inclination);
+	C[0][2] =  sin(RAAN) * sin(Inclination);
 	C[1][0] =  cos(Arg_Of_Perigee) * sin(RAAN) + sin(Arg_Of_Perigee) * cos(RAAN) * cos(Inclination);
 	C[1][1] = -sin(Arg_Of_Perigee) * sin(RAAN) + cos(Arg_Of_Perigee) * cos(RAAN) * cos(Inclination);
+	C[1][2] = -cos(RAAN) * sin(Inclination);
 	C[2][0] =  sin(Arg_Of_Perigee) * sin(Inclination);
 	C[2][1] =  cos(Arg_Of_Perigee) * sin(Inclination);
+	C[2][2] =  cos(Inclination);
 
-	Drag_Term = (((Drag / Mean_Motion) / 3.0) * Elapsed_Time) / 2.0;
+	Drag_Term = ((Drag / Mean_Motion) / 3.0) * Elapsed_Time;
 
 	Orbital_Phase = Mean_Motion * Elapsed_Time * (1.0 + 3.0 * Drag_Term) + (Epoch_MA / 360.0) + Epoch_Orbit;
 
@@ -334,16 +368,22 @@ static void Calculate_Satellite_Position(void)
 	}
 
 	if (Sat_Chosen->Aflag) {
-		Alon = RAD(Sat_Chosen->Alon);
-		Alat = RAD(Sat_Chosen->Alat);
+		if (Sat_Chosen->Minus_Z) {
+			Alon = RAD(Sat_Chosen->Alon + 180.0);
+			Alat = RAD(-Sat_Chosen->Alat);
+		} else {
+			Alon = RAD(Sat_Chosen->Alon);
+			Alat = RAD(Sat_Chosen->Alat);
+		}
 
 		Orbit_X = -cos(Alat) * cos(Alon);
 		Orbit_Y = -cos(Alat) * sin(Alon);
+		Orbit_Z = -sin(Alat);
 
-		Stars_X = Orbit_X * C[0][0] + Orbit_Y * C[0][1];
-		Stars_Y = Orbit_X * C[1][0] + Orbit_Y * C[1][1];
-		Stars_Z = Orbit_X * C[2][0] + Orbit_Y * C[2][1];
-
+		Stars_X = Orbit_X * C[0][0] + Orbit_Y * C[0][1] + Orbit_Z * C[0][2];
+		Stars_Y = Orbit_X * C[1][0] + Orbit_Y * C[1][1] + Orbit_Z * C[1][2];
+		Stars_Z = Orbit_X * C[2][0] + Orbit_Y * C[2][1] + Orbit_Z * C[2][2];
+ 
 		Earth_X = Stars_X *  cos(G7) - Stars_Y * -sin(G7);
 		Earth_Y = Stars_X * -sin(G7) + Stars_Y *  cos(G7);
 		Earth_Z = Stars_Z;		
@@ -358,46 +398,162 @@ static void Calculate_Satellite_Position(void)
 	if (Squint < 0.0) Squint += 180.0;
 }
 
-static void Display_Track_Data(void)
+static void Search_AOS_LOS(struct Sat_Struct *Sat_Chosen, time_t Track_time)
+{
+	time_t t = Track_time, t0, t1, tprev;
+	double e0, e1;
+	int i;
+
+	Calculate_Times(Sat_Chosen, t);
+	Calculate_Satellite_Position(Sat_Chosen);
+
+	for (i = 0; i < 2; i++) {
+		/* first, a coarse search in steps of 5 minutes, to find the next time at which the elevation changes sign */
+		do {
+			e0 = Elevation;
+			t0 = t;
+			t += 300;
+			Calculate_Times(Sat_Chosen, t);
+			Calculate_Satellite_Position(Sat_Chosen);
+		} while ((t < Track_time + 86400 || i > 0) && e0 * Elevation >= 0.0);
+	
+		if (e0 * Elevation >= 0.0) {
+			/* Satellite won't rise or set in the next 24 hours - possibly too low inclination? */
+			Next_AOS = 0;
+			Next_LOS = 0;
+			return;
+		}
+
+		e1 = Elevation;
+		t1 = t;
+		t = t0 - 1;
+
+		/* do a "regula falsi" search to quickly approximate the AOS/LOS time */
+		while (TRUE) {
+			tprev = t;
+			t = 0.5 + (e0 * t1 - e1 * t0) / (e0 - e1);
+
+			if (t == tprev)
+				break;
+
+			Calculate_Times(Sat_Chosen, t);
+			Calculate_Satellite_Position(Sat_Chosen);
+
+			if (e0 * Elevation >= 0.0) {
+				e0 = Elevation;
+				t0 = t;
+			} else {
+				e1 = Elevation;
+				t1 = t;
+			}
+		}
+
+		/* finally, check the AOS/LOS time and correct if necessary */
+		if (t == t0) {
+			do {
+				e0 = Elevation;
+				Calculate_Times(Sat_Chosen, ++t);
+				Calculate_Satellite_Position(Sat_Chosen);
+			} while (e0 * Elevation > 0.0);
+
+			e1 = Elevation;
+			t--;
+		} else {
+			do {
+				e1 = Elevation;
+				t1 = t;
+				Calculate_Times(Sat_Chosen, --t);
+				Calculate_Satellite_Position(Sat_Chosen);
+			} while (e1 * Elevation > 0.0);
+
+			e0 = Elevation;
+		}
+
+		/* round in the correct direction */
+		if (fabs(e0) > fabs(e1)) t++;
+
+		if (e1 > e0)
+			Next_AOS = t;
+		else
+			Next_LOS = t;
+
+		t = t1;
+		Elevation = e1;
+   	}
+}
+
+static char *Format_Date_Time(struct tm *tm)
+{
+	static char Buffer[41];
+
+	switch (User_Data.Date_Format) {
+		case DATE_FORMAT_DDMMYYYY:
+			sprintf(Buffer, "%02d-%02d-%04d %02d:%02d:%02d",
+								tm->tm_mday,
+							        tm->tm_mon + 1,
+							        tm->tm_year + 1900,
+								tm->tm_hour,
+							        tm->tm_min,
+							        tm->tm_sec);
+			break;
+		case DATE_FORMAT_MMDDYYYY:
+			sprintf(Buffer, "%02d/%02d/%04d %02d:%02d:%02d",
+								tm->tm_mon + 1,
+							        tm->tm_mday,
+							        tm->tm_year + 1900,
+								tm->tm_hour,
+							        tm->tm_min,
+							        tm->tm_sec);
+			break;
+		default:
+			g_error("mtrack: unknown date format %d\n", User_Data.Date_Format);
+			return "";
+	}
+
+	return Buffer;
+}
+
+static char *Diff_Time(time_t timesoon)
+{
+	static char Buffer[21];
+	struct tm *tm;
+	double Elapsed_Time;
+	int Days, Hours, Mins, Secs;
+	
+	tm = gmtime(&timesoon);
+
+	Elapsed_Time = ((double)Year_Conversion(tm->tm_year + 1900) +
+			(double)(tm->tm_yday + 1) +
+			(double)tm->tm_hour / 24.0  +
+			(double)tm->tm_min / 1440.0 +
+			(double)tm->tm_sec / (60.0 * 1440.0)) - Track_Day_Time;
+
+	Secs  = (int)(Elapsed_Time * (60.0 * 1440.0));
+	Days  = Secs / (24 * 3600); Secs -= Days * (24 * 3600);
+	Hours = Secs / 3600;        Secs -= Hours * 3600;
+	Mins  = Secs / 60;          Secs -= Mins * 60;
+
+	sprintf(Buffer, "%d/%02d:%02d:%02d", Days, Hours, Mins, Secs);
+
+	return Buffer;
+}
+
+static void Display_Track_Data(struct Sat_Struct *Sat_Chosen, time_t Track_time)
 {
-	static double Prev_Range        = 0.0;
-	static double Prev_Elapsed_Time = 0.0;
-	static double Prev_SSP_Long     = -1.0;
-	static double Prev_SSP_Lat      = -1.0;
-	static double Prev_Half_Angle   = -1.0;
 	double Change_Range;
 	double Change_Time;
 	double Fraction;
-	char Buffer[41];
+	double Doppler;
+	char   Sign;
+	struct tm *tm;
+	char Buffer[81];
 
 	Clear_Text();
 
 	sprintf(Buffer, "Sat: %s", Sat_Chosen->Name);
 	Write_Text(TEXT_SAT, Buffer);
 
-	switch (User_Data.Date_Format) {
-		case DATE_FORMAT_DDMMYYYY:
-			sprintf(Buffer, "Date/Time: %02d-%02d-%04d %02d:%02d:%02d UTC",
-								Current_Time.tm_mday,
-							        Current_Time.tm_mon,
-							        Current_Time.tm_year + 1900,
-								Current_Time.tm_hour,
-							        Current_Time.tm_min,
-							        Current_Time.tm_sec);
-			break;
-		case DATE_FORMAT_MMDDYYYY:
-			sprintf(Buffer, "Date/Time: %02d/%02d/%04d %02d:%02d:%02d UTC",
-								Current_Time.tm_mon,
-							        Current_Time.tm_mday,
-							        Current_Time.tm_year + 1900,
-								Current_Time.tm_hour,
-							        Current_Time.tm_min,
-							        Current_Time.tm_sec);
-			break;
-		default:
-			g_error("mtrack: unknown date format %d\n", User_Data.Date_Format);
-			break;
-	}
+	sprintf(Buffer, "Date/Time: %s UTC", Format_Date_Time(&Current_Time));
 	Write_Text(TEXT_DATETIME, Buffer);
 
 	sprintf(Buffer, "Az: %.0f deg", Azimuth);
@@ -421,49 +577,56 @@ static void Display_Track_Data(void)
 	sprintf(Buffer, "Squint: %.0f deg", Squint);
 	Write_Text(TEXT_SQUINT, Buffer);
 
-	Change_Range = Range - Prev_Range;
-	Change_Time  = (Elapsed_Time - Prev_Elapsed_Time) * 86400.0;
+	Change_Range = Range - Sat_Chosen->Range;
+	Change_Time  = (Elapsed_Time - Sat_Chosen->Elapsed_Time) * 86400.0;
 
-	Fraction = 1.0 - (Change_Range / Change_Time) / 299792.0;
+	Fraction = (Change_Range / Change_Time) / 299792.0;
 
 	if (Sat_Chosen->Frequency1 != 0.0) {
-		sprintf(Buffer, "Frequency 1: %.4f MHz", Fraction * Sat_Chosen->Frequency1);
+		Doppler = -Fraction * Sat_Chosen->Frequency1;
+		Sign    = (Fraction < 0.0) ? '+' : '-';
+	
+		sprintf(Buffer, "Frequency 1: %.4f MHz  %c%.1f kHz  %.4f MHz", Sat_Chosen->Frequency1, Sign, fabs(Doppler * 1000.0), Sat_Chosen->Frequency1 + Doppler);
 		Write_Text(TEXT_FREQ1, Buffer);
 	} else {
-		Write_Text(TEXT_FREQ1, "");
+		Write_Text(TEXT_FREQ1, "Frequency 1: N/A");
 	}
 
 	if (Sat_Chosen->Frequency2 != 0.0) {
-		sprintf(Buffer, "Frequency 2: %.4f MHz", Fraction * Sat_Chosen->Frequency2);
+		Doppler = -Fraction * Sat_Chosen->Frequency2;
+		Sign    = (Fraction < 0.0) ? '+' : '-';
+	
+		sprintf(Buffer, "Frequency 2: %.4f MHz  %c%.1f kHz  %.4f MHz", Sat_Chosen->Frequency2, Sign, fabs(Doppler * 1000.0), Sat_Chosen->Frequency2 + Doppler);
 		Write_Text(TEXT_FREQ2, Buffer);
 	} else {
-		Write_Text(TEXT_FREQ2, "");
+		Write_Text(TEXT_FREQ2, "Frequency 2: N/A");
 	}
 
-	if (Sat_Chosen->Frequency3 != 0.0) {
-		sprintf(Buffer, "Frequency 3: %.4f MHz", Fraction * Sat_Chosen->Frequency3);
-		Write_Text(TEXT_FREQ3, Buffer);
+	if (Next_AOS != 0 && Next_LOS != 0) {
+		if (Next_AOS <= Next_LOS) {
+			tm = gmtime(&Next_AOS);
+			sprintf(Buffer, "Next AOS: %s UTC  %s", Format_Date_Time(tm), Diff_Time(Next_AOS));
+			Write_Text(TEXT_ALOS1, Buffer);
+
+			tm = gmtime(&Next_LOS);
+			sprintf(Buffer, "Next LOS: %s UTC  %s", Format_Date_Time(tm), Diff_Time(Next_LOS));
+			Write_Text(TEXT_ALOS2, Buffer);
+		} else {
+			tm = gmtime(&Next_LOS);
+			sprintf(Buffer, "Next LOS: %s UTC  %s", Format_Date_Time(tm), Diff_Time(Next_LOS));
+			Write_Text(TEXT_ALOS1, Buffer);
+
+			tm = gmtime(&Next_AOS);
+			sprintf(Buffer, "Next AOS: %s UTC  %s", Format_Date_Time(tm), Diff_Time(Next_AOS));
+			Write_Text(TEXT_ALOS2, Buffer);
+		}
 	} else {
-		Write_Text(TEXT_FREQ3, "");
+		Write_Text(TEXT_ALOS1, "Next AOS: N/A");
+		Write_Text(TEXT_ALOS2, "Next LOS: N/A");
 	}
-
-	Prev_Range        = Range;
-	Prev_Elapsed_Time = Elapsed_Time;
-
-	if (fabs(Prev_SSP_Lat -  SSP_Lat)  > 1.0 ||
-	    fabs(Prev_SSP_Long - SSP_Long) > 1.0 ||
-	    fabs(Prev_Half_Angle - Half_Angle) > 1.0) {
-		Plot_Satellite(SSP_Lat, SSP_Long, Half_Angle);
-
-		Prev_SSP_Lat    = SSP_Lat;
-		Prev_SSP_Long   = SSP_Long;
-		Prev_Half_Angle = Half_Angle;
-	}
-
-	Refresh_Text();
 }
 
-static void Display_List_Data(GtkWidget *list)
+static void Display_List_Data(struct Sat_Struct *Sat_Chosen, GtkWidget *list)
 {
 	char DateBuffer[12];
 	char TimeBuffer[12];
@@ -479,11 +642,11 @@ static void Display_List_Data(GtkWidget *list)
 	switch (User_Data.Date_Format) {
 		case DATE_FORMAT_DDMMYYYY:
 			sprintf(DateBuffer, "%02d-%02d-%04d", Current_Time.tm_mday,
-							      Current_Time.tm_mon,
+							      Current_Time.tm_mon + 1,
 							      Current_Time.tm_year + 1900);
 			break;
 		case DATE_FORMAT_MMDDYYYY:
-			sprintf(DateBuffer, "%02d/%02d/%04d", Current_Time.tm_mon,
+			sprintf(DateBuffer, "%02d/%02d/%04d", Current_Time.tm_mon + 1,
 							      Current_Time.tm_mday,
 							      Current_Time.tm_year + 1900);
 			break;
@@ -549,7 +712,7 @@ double DEG(double angle)
 	return (angle / PI) * 180.0;
 }
 
-void Show_Elements(void)
+void Show_Elements(struct Sat_Struct *Sat)
 {
 	GtkWidget *window;
 	GtkWidget *vbox;
@@ -560,11 +723,22 @@ void Show_Elements(void)
 	gchar *title[4];
 	gchar *data[3];
 	double Number;
-	char Buffer[20];
+	time_t timenow;
+	struct tm *tm;
+	char Buffer[50];
+
+	time(&timenow);
+	tm = gmtime(&timenow);
+
+	Calculate_Initial_Data(Sat);
+	Calculate_Times(Sat, timenow);
+	Calculate_Satellite_Position(Sat);
 
 	window = gtk_window_new(GTK_WINDOW_DIALOG);
 
-	gtk_window_set_title(GTK_WINDOW(window), "Updated Elements");
+	sprintf(Buffer, "Updated Elements for %s", Sat->Name);
+
+	gtk_window_set_title(GTK_WINDOW(window), Buffer);
 	gtk_container_set_border_width(GTK_CONTAINER(window), BORDER_WIDTH);
 
 	vbox = gtk_vbox_new(FALSE, BORDER_WIDTH);
@@ -604,11 +778,11 @@ void Show_Elements(void)
 
 	data[0] = "Epoch Date";
 	sprintf(Buffer, "%.6f",
-		 (double)Current_Time.tm_year * 1000.0 + 1900000.0 +
-		 (double)Current_Time.tm_yday +
-		 (double)Current_Time.tm_hour / 24.0  +
-		 (double)Current_Time.tm_min / 1440.0 +
-		 (double)Current_Time.tm_sec / (60.0 * 1440.0));
+		 (double)tm->tm_year * 1000.0 + 1900000.0 +
+		 (double)(tm->tm_yday + 1) +
+		 (double)tm->tm_hour / 24.0  +
+		 (double)tm->tm_min / 1440.0 +
+		 (double)tm->tm_sec / (60.0 * 1440.0));
 	data[1] = Buffer;
 	data[2] = "";
 	gtk_clist_append(GTK_CLIST(list), data);
@@ -620,7 +794,7 @@ void Show_Elements(void)
 	gtk_clist_append(GTK_CLIST(list), data);
 
 	data[0] = "Inclination";
-	sprintf(Buffer, "%.4f", Sat_Chosen->Inclination);
+	sprintf(Buffer, "%.4f", Sat->Inclination);
 	data[1] = Buffer;
 	data[2] = "deg";
 	gtk_clist_append(GTK_CLIST(list), data);
@@ -635,7 +809,7 @@ void Show_Elements(void)
 	gtk_clist_append(GTK_CLIST(list), data);
 
 	data[0] = "Eccentricity";
-	sprintf(Buffer, "%.7f", Sat_Chosen->Eccentricity);
+	sprintf(Buffer, "%.7f", Sat->Eccentricity);
 	data[1] = Buffer;
 	data[2] = "";
 	gtk_clist_append(GTK_CLIST(list), data);
@@ -659,13 +833,13 @@ void Show_Elements(void)
 	gtk_clist_append(GTK_CLIST(list), data);
 
 	data[0] = "Mean Motion";
-	sprintf(Buffer, "%.8f", Sat_Chosen->Mean_Motion);
+	sprintf(Buffer, "%.8f", Sat->Mean_Motion);
 	data[1] = Buffer;
 	data[2] = "revs/day";
 	gtk_clist_append(GTK_CLIST(list), data);
 
 	data[0] = "Decay Rate";
-	sprintf(Buffer, "%E", Sat_Chosen->Drag);
+	sprintf(Buffer, "%E", Sat->Drag);
 	data[1] = Buffer;
 	data[2] = "revs/day/day";
 	gtk_clist_append(GTK_CLIST(list), data);
@@ -677,7 +851,7 @@ void Show_Elements(void)
 	gtk_clist_append(GTK_CLIST(list), data);
 
 	data[0] = "Catalogue No";
-	sprintf(Buffer, "%ld", Sat_Chosen->Catalogue_No);
+	sprintf(Buffer, "%ld", Sat->Catalogue_No);
 	data[1] = Buffer;
 	data[2] = "";
 	gtk_clist_append(GTK_CLIST(list), data);
@@ -717,7 +891,7 @@ void Show_Elements(void)
 	gtk_widget_show(window);
 }
 
-void Show_Locations(void)
+void Show_Locations(struct Sat_Struct *Sat)
 {
 	GtkWidget *window;
 	GtkWidget *vbox;
@@ -734,10 +908,13 @@ void Show_Locations(void)
 	double Range;
 	double Latitude;
 	double Longitude;
+	char   Buffer[40];
 	char   AzBuffer[15];
 	char   ElBuffer[15];
 	char   RaBuffer[15];
 	gchar *data[4];
+	time_t timenow;
+	struct tm *tm;
 	GSList *Loc;
 	struct Loc_Struct *Data;
 
@@ -746,9 +923,18 @@ void Show_Locations(void)
 		return;
 	}
 
+	time(&timenow);
+	tm = gmtime(&timenow);
+
+	Calculate_Initial_Data(Sat);
+	Calculate_Times(Sat, timenow);
+	Calculate_Satellite_Position(Sat);
+
 	window = gtk_window_new(GTK_WINDOW_DIALOG);
 
-	gtk_window_set_title(GTK_WINDOW(window), "Visibility");
+	sprintf(Buffer, "Visibilty for %s", Sat->Name);
+
+	gtk_window_set_title(GTK_WINDOW(window), Buffer);
 	gtk_container_set_border_width(GTK_CONTAINER(window), BORDER_WIDTH);
 
 	vbox = gtk_vbox_new(FALSE, BORDER_WIDTH);
